;********************************************************************
; 17A_10F.asm
;--------------------------------------------------------------------
;
;    Titolo        :  Corso Assembly & C - Esercitazione 17A
;                     Lampeggia un LED collegato a GP2.
;                     ton=20ms  toff=2,3s in sleep per basso consumo.

;    PIC           :  10F200/2/4/6
;    Supporto      :  MPASM
;    Versione      :  1.0
;    Data          :  01-05-2013
;    Ref. hardware :
;    Autore        :  afg
;
;
;  Impiego pin :
;  ----------------
;     10F200/2/4/6 @ 8 pin DIP       10F200/2/4/6 @ 6 pin SOT-23
;
;               |\/|                  *|
;           NC -|1    8|- GP3        GP0 -|1    6|- GP3
;          Vdd -|2    7|- Vss        Vss -|2    5|- Vdd
;          GP2 -|3    6|- NC         GP1 -|3    4|- GP2
;          GP1 -|4    5|- GP0             |______|
;               |______|
;
;                           DIP SOT
;    NC                      1:
;    Vdd                     2:  5: +
;    GP2/T0CKI/FOSC4/[COUT]  3:  4: Out LED alla Vss
;    GP1/[CIN-]/ICSPCLK      4:  3:
;    GP0/[CIN+]/ICSPDAT      5:  1:
;    NC                      6:
;    Vss                     7:  2: -
;    GP3/MCLR/VPP            8:  6:
;
;    [] solo 204/6
;
;********************************************************************
;            DEFINIZIONE DI IMPIEGO DEI PORT
;GPIO map
; |  3  |  2  |  1  |  0  |
; |-----|-----|-----|-----|
; | in  | LED |     |     |
;
;#define    GPIO,GP0   ;
;#define    GPIO,GP1   ;
#define    LED GPIO,GP2   ; LED tra pin e Vss
;#define    GPIO,GP3   ;
;
;####################################################################
; Definizione del processore
 #ifdef __10F200
        LIST      p=10F200
        #include <p10F200.inc>
 #endif
 #ifdef __10F202
        LIST      p=10F202
        #include <p10F202.inc>
 #endif
 #ifdef __10F204
        LIST      p=10F204
        #include <p10F204.inc>
#define  proccomp
 #endif
 #ifdef __10F206
        LIST      p=10F206
        #include <p10F206.inc>
#define  proccomp
 #endif
        radix     dec          ; default numeri decimali

;####################################################################
;                          CONFIGURAZIONE
;
; No WDT, no CP, pin4=GP3;
  __config  _CP_OFF & _MCLRE_OFF  & _WDT_ON

;####################################################################
;                          RAM
    CBLOCK 0x10
    d1,d2,d3             ; counter per temporizzazione
    endc

step EQU  .10            ; ripetizioni lampeggio

;####################################################################
;                           RESET ENTRY
;
; Reset Vector
RESET_VECTOR   ORG     0x00

        clrwdt
; calibrazione oscillatore interno
        andlw   0xFE
        movwf   OSCCAL
; disabilita T0CKI per avere GP2 libero
; prescaler 1:128 al WDT
; OPTION def '11111111'
;             1------- GPWU disabilitato
;             -1------ GPPU disabilitato
;             --0----- clock interno (no T0CKI)
;             ---1---- done
;             ----1--- prescaler al WDT
;             -----111 1:128  2.3s circa
     movlw  b'11011111'
     OPTION

 #ifdef  proccomp   ; solo 10F204/6
; disabilita comparatore
     movlw  b'11110111'
     movwf  CMCON0
 #endif

; GP2 out
     movlw   b'11111011'
     tris    GPIO

     movlw   step
     movwf   d3
ll   bsf     LED
     call    Delay1ms
     bcf     LED
     call    Delay1ms
     decfsz  d3
      goto   ll

     sleep


; subroutines
Delay1ms
        movlw   0xC6
        movwf   d1
        movlw   0x01
        movwf   d2
Delay1ms_0
        decfsz  d1, f
         goto   $+2
        decfsz  d2, f
         goto   Delay1ms_0
        goto    $+1
        nop
        retlw   0
;********************************************************************
;                            THE END
        END

