;********************************************************************
;--------------------------------------------------------------------
;
;    Titolo        :  Corso Assembly & C - Esercitazione 17A
;                     Lampeggia un LED collegato a GP2.
;                     ton=100ms  toff=1s per basso consumo.

;    PIC           :  10F200/2
;    Supporto      :  MPASM
;    Versione      :  1.0
;    Data          :  01-05-2013
;    Ref. hardware :
;    Autore        :  afg
;
;
;  Impiego pin :
;  ----------------
;     10F200/202 @ 8 pin DIP       10F200/202 @ 6 pin SOT-23
;
;               |\/|                  *|
;           NC -|1    8|- GP3        GP0 -|1    6|- GP3
;          Vdd -|2    7|- Vss        Vss -|2    5|- Vdd
;          GP2 -|3    6|- NC         GP1 -|3    4|- GP2
;          GP1 -|4    5|- GP0             |______|
;               |______|
;
;                       DIP  SOT
;    NC                  1:     nc
;    Vdd                 2:  5: ++
;    GP2/T0CKI/FOSC4     3:  4: Out LED alla Vss
;    GP1/ICSPCLK         4:  3:
;    GP0/ICSPDAT         5:  1:
;    NC                  6:     nc
;    Vss                 7:  2: --
;    GP3/MCLR/VPP        8:  6:
;
;********************************************************************
;            DEFINIZIONE DI IMPIEGO DEI PORT
;GPIO map
; |  3  |  2  |  1  |  0  |
; |-----|-----|-----|-----|
; | in  | LED |     |     |
;
;#define    GPIO,GP0   ;
;#define    GPIO,GP1   ;
#define    LED GPIO,GP2   ; LED tra pin e Vss
;#define    GPIO,GP3   ;
;
;####################################################################
 #ifdef __10F200
        LIST      p=10F200     ; definizione del processore
        #include <p10F200.inc>
 #endif
 #ifdef __10F202
        LIST      p=10F202     ; definizione del processore
        #include <p10F202.inc>
 #endif
 #ifdef __10F204
        LIST      p=10F204
        #include <p10F204.inc>
#define  proccomp
 #endif
 #ifdef __10F206
        LIST      p=10F206
        #include <p10F206.inc>
#define  proccomp
 #endif
        radix     dec          ; default numeri decimali

;####################################################################
;                          CONFIGURAZIONE
;
; No WDT, no CP, pin4=GP3;
  __config  _CP_OFF & _MCLRE_OFF  & _WDT_ON

;####################################################################
;                          RAM
    CBLOCK 0x10
    d3             ; counter per temporizzazione

;####################################################################
;                           RESET ENTRY
;
; Reset Vector
RESET_VECTOR   ORG     0x00


;####################################################################
;                        MAIN PROGRAM
;
Main
        btfss   STATUS,NOT_PD    ; Test per POR
         goto   wdtres           ; no

POR_res  ; reset da alimentazione
        clrwdt
        call    IOinit           ; inizializza I/O
reload  movlw   b'11011000'      ; prescaler 1:1 18ms
        option
        movlw   .50              ; carica contatore
        movwf   d3
        bsf     LED              ; accende LED
        goto    $                ; arresta in attesa del WDT

wdtres  clrwdt
        call    IOinit           ; inizializza I/O
        movf    d3,w             ; contatore step azzerato ?
        bz      reload           ; si - ricarica
        decfsz  d3,f             ; no - contatore-1 =0?
         goto   blink            ; no - lampeggi
;si - cambia prescaler per 1s
        movlw   b'11011110'      ; prescaler 1:64
        option
        sleep

; lampeggi rapidi
blink   movlw   LEDpin
        xorlw   GPIO,f


; subroutines

IOinit         ; inizializzazioni al reset
; calibrazione oscillatore interno
        andlw   0xFE
        movwf   OSCCAL
; disabilita T0CKI per avere GP2 libero
; prescaler 1:64 al WDT
; OPTION def '11111111'
;             1------- GPWU disabilitato
;             -1------ GPPU disabilitato
;             --0----- clock interno (no T0CKI)
;             ---1---- done
;             ----1--- prescaler al WDT
;             -----000 1:1
     movlw  b'11011110'
     OPTION

 #ifdef #define  proccomp   ; solo 10F204/6
; disabilita comparatore
     movlw  b'11110111'
     movwf  COMCON0
 #endf

; GP2 out
     movlw   b'11111010'
     tris    GPIO

;********************************************************************
;                            THE END
        END

