;********************************************************************
; 17A_526.asm
;--------------------------------------------------------------------
;
;    Titolo        : Corso Assembly & C - Esercitazione 17A
;                    Misura indiretta della Vdd con risultato ADC
;                    in volt su due cifre
;
;    PIC           : 16F506/526
;    Supporto      : MPASM
;    Versione      : V.519-1.0
;    Data          : 01-05-2013
;    Ref. hardware :
;    Autore        :  afg
;
;--------------------------------------------------------------------
;
;  Impiego pin :
;  ----------------
;       16F506/526 @ 14 pin
;
;                  |\/|
;             Vdd -|1   14|- Vss
;             RB5 -|2   13|- RB0
;             RB4 -|3   12|- RB1
;        RB3/MCLR -|4   11|- RB22
;             RC5 -|5   10|- RC0
;             RC4 -|6    9|- RC1
;             RC3 -|7    8|- RC2
;                  |______|
;
;    Vdd                    1: ++
;    RB5/OSC1/CLKIN         2: Out  segm g
;    RB4/OSC2/CLKOUT        3:
;    RB3/!MCLR/VPP          4: MCLR
;    RC5/T0CKI              5: Out  segm f
;    RC4/C2OUT              6: Out  segm e
;    RC3                    7: Out  segm d
;    RC2/CVref              8: Out  segm c
;    RC1/C2IN-              9: Out  segm b
;    RC0/C2IN+             10: Out  segm a
;    RB2/C1OUT/AN2         11: Out  gate unit
;    RB1/C1IN-/AN1/ICSPC   12: Out  gate decine
;    RB0/C1IN+/AN0/ICSPD   13:
;    Vss                   14: --
;
;####################################################################
; scelta del processore
  #ifdef  __16F526
         LIST      p=16F526
         #include <p16F526.inc>
  #endif
  #ifdef  __16F506
        LIST      p=16F506
         #include <p16F506.inc>
  #endif

        radix  dec

;####################################################################
;                          CONFIGURAZIONE
  #ifdef  __16F526
; Oscillatore interno, 4MHz, no WDT, no CP, MCLR
 __config  _IntRC_OSC_RB4 & _IOSCFS_4MHz & _WDTE_OFF & _CP_OFF & _CPDF_OFF & _MCLRE_ON
  #endif
  #ifdef  __16F506
; Oscillatore interno, 4MHz, no WDT, no CP, MCLR
  __config  _IntRC_OSC_RB4EN & _IOSCFS_OFF & _WDT_OFF & _CP_OFF & _MCLRE_ON
  #endif

;####################################################################
;                           MEMORIA RAM
; general purpose RAM
    UDATA                   ; area RAM
temp    res 1               ; temporaneo
Vtemp   res 1               ; temporaneo ADC
cntr    res 1               ; counter


 ;********************************************************************
;====================================================================
;            DEFINIZIONE DI IMPIEGO DEI PORT
;
;PORTC map
;|  5  |  4  |  3  |  2  |  1  |  0  |
;|-----|-----|-----|-----|-----|-----|
;|segmf|segme|segmd|segmc|segmb|segma|
;
#define     segma  PORTC,0    ; segmenti f:a
#define     segmb  PORTC,1    ;
#define     segmc  PORTC,2    ;
#define     segmd  PORTC,3    ;
#define     segme  PORTC,4    ;
#define     segmf  PORTC,5    ;

;PORTB map
;|  5  |  4  |  3  |  2  |  1  |  0  |
;|-----|-----|-----|-----|-----|-----|
;|segmg|     |     | AN2 |GATE1|GATE2|
;
;#define           PORTB,0    ;
#define     GATE1  PORTB,1    ; gate MOSFET cifra decine
#define     GATE2  PORTB,2    ; gate MOSFET cifra unit
;#define           PORTB,3    ; MCLR
#define     segmdp PORTB,4    ; dp
#define     segmg  PORTB,5    ; segmento g
;
;####################################################################
;                         COSTANTI
;
#define  bit05k  TMR0,1 ; bit TMR0 per 512us
#define  bit1k   TMR0,2 ;             1024us
#define  bit2k   TMR0,3 ;             2048us
#define  bit4k   TMR0,4 ;             4096us
#define  bit8k   TMR0,5 ;             8192us

#define bittmr   bit4k  ; 4096us
tempo   EQU 0           ; numero di ccli tra le conversioni
                        ; 256 x 4096us = 1s

; limiti risultato ADC
 constant min = 0x1B
 constant max = 0x4C

;####################################################################
;                         MACRO LOCALI
; comando gate MOSFET
UNITon   MACRO
        bsf GATE2
          ENDM
UNIToff  MACRO
        bcf GATE2
          ENDM
DECon   MACRO
        bsf GATE1
          ENDM
DECoff  MACRO
        bcf GATE1
          ENDM

; macro generali
 #include C:\PIC\Library\Baseline\basemacro.asm
 #include C:\PIC\Library\Baseline\compmacro.asm

;####################################################################
;                         RESET ENTRY
;
; Reset Vector
        CODE     0x00

; calibrazione oscillatore interno
        movwf   OSCCAL

; salto per lasciare spazio ai vettori indiretti per le subroutines
        pagesel start
        goto    start

;-------------------------------------------------------------------
; vettori subroutines
ADConv:                   ; conversione AD
        pagesel ADConv_r
        goto    ADConv_r
volttbl:                  ; lookup table per le tensioni
        pagesel volttbl_r
        goto    volttbl_r
segtbl:                   ; lookup table maschere dei segmenti
        pagesel segtbl_r
        goto    segtbl_r

;####################################################################
;                        MAIN PROGRAM
Main     CODE

start:
; disabilita comparatori per liberare la funzione digitale
        bcf     CM1CON0, C1ON
        bcf     CM2CON0, C2ON

; disabilita T0CKI da RB5, prescaler 1:256 al Timer0
         ;      b'11010111'
         ;        1-------     GPWU abilitato
         ;        -1------     GPPU abilitato
         ;        --0-----     clock interno
         ;        ---1----     falling
         ;        ----0---     prescaler al Timer0
         ;        -----111     1:256
        movlw   b'11010111'
        OPTION

        clrf    PORTB     ; clear latch del port
        clrf    PORTC

; Tutti i port utili come out
        movlw   0
        tris    PORTB
        tris    PORTC

; ADC per misura riferimento interno
                ; 00------  no ingressi analogici
                ; --11----  INTOSC/4
                ; ----11--  riferimento interno
                ; ------0-  GO/Done
                ; -------1  abilita ADC
        movlw   b'00111101'
        movwf   ADCON0

        movlw   tempo     ; contatore cadenza conversioni
        movwf   cntr

        clrf    TMR0      ; inizializza timer

;-------------------------------------------------------------------
; ciclo display
; digit unit
displp  btfss   bittmr     ; 2.048ms ?
         goto   displp     ; no - attesa
        movf    Vtemp,w    ; si - carica risultato conversione
        andlw   0x0F       ; solo nibble basso
        pagesel segtbl
        call    segtbl     ; lookup table
        pagesel $
        movwf   temp       ; salva in temporaneo
        movwf   PORTC      ; passa al port segm f:a
        btfsc   temp,6     ; comanda segmento g
         bsf    segmg
        btfss   temp,6
         bcf    segmg
        UNITon             ; cifra unit accesa
dslp_1  btfsc   bittmr     ; 2.048ms ?
         goto   dslp_1     ; no - attesa
dslp_2  UNIToff            ; si - spegni unit
; digit decine
; swap Vtemp in W, nibble basso<->nibble alto
        swapf   Vtemp,w
        andlw   0x0F       ; solo nibble basso
        pagesel segtbl
        call    segtbl     ; lookup table
        pagesel $
        movwf   temp
        movwf   PORTC
        btfsc   temp,6
         bsf     segmg
        btfss   temp,6
         bcf     segmg
        bsf     segmdp     ; aggiungi punto decimale
        DECon              ; accendi decine

; conversione AD
        decfsz  cntr,f     ; acquisisci solo ogni tempo * bittmr
         goto   dslp_3     ; 256 * 4ms = 1s circa
        movlw   tempo      ; ricarica contatore
        movwf   cntr
        pagesel ADConv
        call    ADConv     ; conversione AD
        movf    ADRES,w
        movwf   Vtemp
        incf    Vtemp,f    ; correzione +1
        pagesel volttbl
        call    volttbl    ; tabella per valore in volt
        pagesel $
        movwf   Vtemp      ; salva

; fine tempo ?
dslp_3  btfss   bittmr     ; 2.048ms ?
         goto   dslp_3     ; no - attesa
        DECoff             ; si - spegni decine
        bcf     segmdp     ; spegni punto decimale
        goto    displp     ; loop


TABLES  CODE  0x200
;####################################################################
;             AREA TABELLE E SUBROUTINES in PAGINA1
;---------------------------------------------------------------------
; lookup table per convertire ADC da hex in volt
volttbl_r:
   CFLJB   Vtemp, min, aderL  ; se ADC<minimo, errore LL
   CFLJA   Vtemp, max, aderH  ; se ADC>massimo, errore HH
   movlw   min                ; parti da 0 nella tabella
   subwf   Vtemp,w
   addwf   PCL,f              ; punta PC
;               #     ADC
   retlw   0x57  ;0   1B
   retlw   0x55  ;1   1C
   retlw   0x53  ;2   1D
   retlw   0x51  ;3   1E
   retlw   0x50  ;4   1F
   retlw   0x48  ;5   20
   retlw   0x47  ;6   21
   retlw   0x45  ;7   22
   retlw   0x44  ;8   23
   retlw   0x43  ;9   24
   retlw   0x42  ;10  25
   retlw   0x40  ;11  26
   retlw   0x39  ;12  27
   retlw   0x38  ;13  28
   retlw   0x37  ;14  29
   retlw   0x37  ;15  2A
   retlw   0x36  ;16  2B
   retlw   0x35  ;17  2C
   retlw   0x34  ;18  2D
   retlw   0x33  ;19  2E
   retlw   0x33  ;20  2F
   retlw   0x32  ;21  30
   retlw   0x31  ;22  31
   retlw   0x31  ;23  32
   retlw   0x30  ;24  33
   retlw   0x30  ;25  34
   retlw   0x29  ;26  35
   retlw   0x28  ;27  36
   retlw   0x28  ;28  37
   retlw   0x27  ;29  38
   retlw   0x27  ;30  39
   retlw   0x26  ;31  3A
   retlw   0x26  ;32  3B
   retlw   0x26  ;33  3C
   retlw   0x25  ;34  3D
   retlw   0x25  ;35  3E
   retlw   0x24  ;36  3F
   retlw   0x24  ;37  40
   retlw   0x24  ;38  41
   retlw   0x23  ;39  42
   retlw   0x23  ;40  43
   retlw   0x23  ;41  44
   retlw   0x22  ;42  45
   retlw   0x22  ;43  46
   retlw   0x22  ;44  47
   retlw   0x21  ;45  48
   retlw   0x21  ;46  49
   retlw   0x21  ;47  4A
   retlw   0x20  ;48  4B
   retlw   0x20  ;49  4C

aderL retlw  0xAA   ; display LL
aderH retlw  0xBB   ; display HH

;---------------------------------------------------------------------
; segment data table - display catodo comune
segtbl_r
        andlw  0x0F          ; solo nibble basso
        addwf  PCL,f         ; punta PC
        retlw  b'00111111'   ; "0" -|-|F|E|D|C|B|A
        retlw  b'00000110'   ; "1" -|-|-|-|-|C|B|-
        retlw  b'01011011'   ; "2" -|G|-|E|D|-|B|A
        retlw  b'01001111'   ; "3" -|G|-|-|D|C|B|A
        retlw  b'01100110'   ; "4" -|G|F|-|-|C|B|-
        retlw  b'01101101'   ; "5" -|G|F|-|D|C|-|A
        retlw  b'01111101'   ; "6" -|G|F|E|D|C|-|A
        retlw  b'00000111'   ; "7" -|-|-|-|-|C|B|A
        retlw  b'01111111'   ; "8" -|G|F|E|D|C|B|A
        retlw  b'01101111'   ; "9" -|G|F|-|D|C|B|A
        retlw  b'00111000'   ; "L" -|-|F|E|D|-|-|- exit per Ah
        retlw  b'01110110'   ; "H" -|G|F|E|-|C|B|- exit per Bh

;---------------------------------------------------------------------
; Misura tensione
ADConv_r:
; attesa riduzione rumore
        DLY10US
; avvia conversione
        bsf     ADCON0,GO
adlp    btfsc   ADCON0,NOT_DONE
         goto   adlp
        retlw   0

;********************************************************************
;                           THE END
        END
