;********************************************************************
; 17A_10FVdd.asm
;--------------------------------------------------------------------
;
;    Titolo        :  Corso Assembly & C - Esercitazione 17A
;                     Valutazione della Vdd attraverso la misura
;                     indiretta della tensione di riferimento del
;                     modulo ADC.

;    PIC           :  10F220/222
;    Supporto      :  MPASM
;    Versione      :  1.0
;    Data          :  01-05-2013
;    Ref. hardware :
;    Autore        :  afg
;
;
;  Impiego pin :
;  ----------------
;     10F220/222 @ 8 pin DIP       10F220/222 @ 6 pin SOT-23
;
;               |\/|                  *|
;           NC -|1    8|- GP3        GP0 -|1    6|- GP3
;          Vdd -|2    7|- Vss        Vss -|2    5|- Vdd
;          GP2 -|3    6|- NC         GP1 -|3    4|- GP2
;          GP1 -|4    5|- GP0             |______|
;               |______|
;
;                       DIP  SOT
;    NC                  1:     nc
;    Vdd                 2:  5: ++
;    GP2/T0CKI/FOSC4     3:  4: Out LED
;    GP1/ICSPCLK/AN1     4:  3: Out LED
;    GP0/ICSPDAT/AN0     5:  1: Out LED
;    NC                  6:     nc
;    Vss                 7:  2: --
;    GP3/MCLR/VPP        8:  6:
;
;********************************************************************
;            DEFINIZIONE DI IMPIEGO DEI PORT
;GPIO map
; |  3  |  2  |  1  |  0  |
; |-----|-----|-----|-----|
; |     |LED4 |LED3 |LED1 |
;
#define    LED1  GPIO,GP0   ; LED tra pin e Vss
#define    LED3  GPIO,GP1   ;
#define    LED4  GPIO,GP2   ;
;#define    GPIO,GP3   ;
;
;####################################################################
 #ifdef __10F220
        LIST      p=10F220     ; definizione del processore
        #include <p10F220.inc>
 #endif
 #ifdef __10F222
        LIST      p=10F222     ; definizione del processore
        #include <p10F222.inc>
 #endif

        radix     dec          ; default numeri decimali

;####################################################################
;                          CONFIGURAZIONE
;
; 4MHz, no WDT, no CP, pin4=GP3;
  __config  _IOFSCS_4MHZ  & _CP_OFF & _MCLRE_OFF  & _WDT_OFF

;####################################################################
;                          RAM

;####################################################################
;                          COSTANTI
;limiti
 CONSTANT  min = 0x33   ; 3.0V
 CONSTANT  max = 0x22   ; 4.5V

;####################################################################
;                         MACRO
; set macro per comparazione
 #include C:\PIC\Library\Baseline\compmacro.asm

;####################################################################
;                           RESET ENTRY
;
; Reset Vector
RESET_VECTOR   ORG     0x00


;####################################################################
;                        MAIN PROGRAM
;
Main:
; calibrazione oscillatore interno
        andlw   0xFE
        movwf   OSCCAL

; disabilita T0CKI per avere GP2 libero
; prescaler 1:256 al Timer0
; OPTION def '11111111'
;             1------- GPWU disabilitato
;             -1------ GPPU disabilitato
;             --0----- clock interno (no T0CKI)
;             ---1---- done
;             ----0--- prescaler al Timer0
;             -----111 1:256
     movlw  b'11011111'
     OPTION

; ADC per misura riferimento interno
            ; 00------  no ingressi analogici
            ; --11----  INTOSC/4
            ; ----11--  riferimento interno
            ; ------0-  GO/Done
            ; -------1  abilita ADC
     movlw  b'00111101'
     movwf  ADCON0

; GP out
     movlw  b'11111000'
     tris   GPIO

mainloop:
; ritardo 64ms
     movlw  .6
     movwf  TMR0
ll1  movf   TMR0,w
     bnz    ll1

; avvia conversione
     bsf    ADCON0,GO
adlp btfsc  ADCON0,NOT_DONE
      goto  adlp

; ADRES > min? (Vdd>3.3V ?)
     CFLJA  ADRES, min, next1
; ADRES < min
     movlw  b'001'       ; LED 1 acceso
     movwf  GPIO
     goto   mainloop

; ADRES > max? (Vdd>4.5V ?)
next1
     CFLJA  ADRES, max, next2
; min<ADRES<max
     movlw  b'010'       ; LED 3 acceso
     movWf  GPIO
     goto   mainloop

; ADRES>max
next2
     movlw  b'100'       ; LED 4 acceso
     movwf  GPIO
     goto   mainloop


;********************************************************************
;                            THE END
        END

